subroutine spline_spec(nknots, nf, df, freq, cte, slope, lambda,    &
                       qispec, qispec2, qislop, qi2der)

!********************************************************************

   implicit none

   integer, intent(in)                      :: nknots, nf
   real(4), intent(in)                      :: df, lambda
   real(4), dimension(nf),   intent(in)     :: freq
   real(4), dimension(nf),   intent(in)     :: cte, slope

   real(4), dimension(nf),   intent(out)    :: qispec, qispec2
   real(4), dimension(nf),   intent(out)    :: qislop, qi2der

   integer                           :: i,j
   real(4)                           :: basis, dbasis, abasis
   real(4), dimension(nknots)        :: t, alpha
   real(4), dimension(nf,nknots)     :: B0, B1, B2
   real(4), dimension(nknots,nknots) :: H 

!  NNLS matrices, QR matrices

   integer                                :: m, n, mode , k
   real(8), dimension(nknots,nknots)      :: H8, U, U_T, E, H_exp
   real(8), dimension(nknots,nknots)      :: L
   real(8), dimension(nknots)             :: hl  
   real(8), dimension(2*nf+nknots,nknots) :: A
   real(8), dimension(2*nf+nknots)        :: b
   real(8), dimension(nknots)             :: alpha8, w
   real(8)                                :: rnorm   !, lambda
   integer, dimension(nknots)             :: indx

   real(4), dimension(2*nf+nknots,2*nf+nknots) :: Q_T
   real(4), dimension(2*nf+nknots,nknots)      :: Q, Aqr
   real(4), dimension(2*nf+nknots)             :: bqr, btilde
   real(4), dimension(nknots,nknots)           :: R


!********************************************************************

   if (nknots /= nf+2) then
      write(6,*) 'Number of knots not valid '
      stop
   endif

   t = real( (/(i-2, i=1,nknots)/) )*df

   B0     = 0.
   B1     = 0.
   B2     = 0.
   H      = 0.

   do i=1,nf

      do j = 1,nknots
         B0(i,j) =  basis(t(j),freq(i),df) 
         B1(i,j) = dbasis(t(j),freq(i),df)/df 
         B2(i,j) = abasis(t(j),freq(i),df)/(df**2) 
      enddo
         
   enddo

   call pen_matrix(nknots,H)

!  Get eigenvalues and eigenvectors of Gram matrix.

   H8 = dble(H)

   call rsm_eig( nknots, H8, nknots, hl, U )

   U_T = transpose(U)

   write(6,*) 'Largest 6 eigenvalues of Gram matrix', hl(1:6)
   if (minval(hl) < 0.0d0) then
      write(6,*) 'Smallest eigenvalue (should be > 0.0 ', minval(hl)
   endif

   E = 0.0d0
   H_exp    = 0.0d0
   do i = 1,nknots
      E(i,i)        = sqrt(hl(i))
      H_exp(i,i)    = hl(i)
   enddo

   H_exp = matmul(U,H_exp)
   H_exp = matmul(H_exp,U_T) 
   L     = matmul(U,E)	! Create sqrt of Gram matrix
   L     = matmul(L,U_T)

   if (maxval(abs(H - real(H_exp))) > 1.d-10) then
      write(6,*) 'Eigenrepresentation error of Gram matrix 1 ', &
                  maxval(abs(H - real(H_exp)))
      stop
   endif

   H_exp = matmul(L,L)
   if (maxval(abs(H - real(H_exp))) > 1.d-10) then
      write(6,*) 'Eigenrepresentation error of Gram matrix 2 ', &
                  maxval(abs(H - real(H_exp)))
      stop
   endif

!
!  The NNLS solution
!

   m = 2*nf+nknots
   n = nknots
!   lambda = 10.0d0

   do i = 1,nf
      A(i,:)                 = dble(sum(cte))/dble(nf) * dble(B0(i,:))
      A(nf+i,:)              = dble(sum(cte))/dble(nf) * dble(B1(i,:))
      Aqr(i,:)               = B0(i,:)
      Aqr(nf+i,:)            = 1./1. *B1(i,:)
   enddo
   A(2*nf+1:2*nf+nknots,:)   = dble(lambda)*L(:,:)
   Aqr(2*nf+1:2*nf+nknots,:) = lambda * L(:,:)

   b(1:nf)                 = dble(sum(cte))/dble(nf) * dble(cte(:))
   b(nf+1:2*nf)            = dble(sum(cte))/dble(nf) * dble(slope(:)) 
   b(2*nf+1:m)             = 0.0d0

   bqr(1:nf)                 = log(cte(:))
   bqr(nf+1:2*nf)            = 1./1.* (slope(:)/cte(:)) 
   bqr(2*nf+1:m)             = 0.0
!
!  If Log domain, use QR decomposition

   call qrfac(m,n,Aqr,Q,R)

   Q_T = transpose(Q)

   btilde = matmul(Q_T,bqr)

   alpha = 0.0

   do i = n, 1, -1
      do k = i+1, n
         alpha(i) = alpha(i) - R(i,k)*alpha(k)
      enddo
      alpha(i) = (alpha(i) + btilde(i))/R(i,i)
   enddo


   qispec2 = matmul(B0,alpha)
!   qispec2 = exp(qispec2)
!   qislop = matmul(B1,alpha)
!   qi2der = matmul(B2,alpha)


!  If linear domain, use NNLS

   call nnls(A,m,n,b, alpha8,rnorm,w,indx,mode)
   
   if (mode /= 1) then
      write(6,*) 'Error in NNLS inversion, code = ', mode
   endif

   alpha = real(alpha8)

!  Generate spectrum with Spline representation

   qispec = matmul(B0,alpha)
   qislop = matmul(B1,alpha)
   qi2der = matmul(B2,alpha)

   w = matmul(H,alpha)
   print *, 'Second derivative norm ', sum(alpha*w)

   print *, 'Misfit norm ', sum(cte - qispec) + sum(slope - qislop)  

end subroutine spline_spec


!--------------------------------------------------------------------

real function basis(tl,timej,deltat)

!********************************************************************

   implicit real (a-h,o-z)

!********************************************************************

   tt=(timej-tl)/deltat

   if (-2.0.le.tt .and. -1.0.gt.tt)then
      tt=tt+2.
      basis= (1./6.) * tt*tt*tt
   elseif (-1.0.le.tt .and. 0.0.gt.tt)then
      tsq=tt*tt
      basis= 2./3. - tt*tt - 0.5*tsq*tt
   elseif (0.0.le.tt .and. 1.0.gt.tt) then
      tsq=tt*tt
      basis= 2./3. - tt*tt + 0.5*tt*tsq
   elseif(1.0.le.tt .and. 2.0.ge.tt)then
      tt=2.-tt
      basis= (1./6.) * tt*tt*tt
   else
      basis=0.
   endif
   
   return
end

!--------------------------------------------------------------------

real function dbasis(tl,timej,deltat)

!
!  finds derivative of cubic spline basis function for knot tl at timej,
!  with equal spacing deltat between knots
!
!**********calls no others
!

!********************************************************************

   implicit real (a-h,o-z)

!********************************************************************

   tt=(timej-tl)/deltat

   if (-2.0.le.tt.and.-1.0.gt.tt)then
      tt=tt+2.
      dbasis= 0.5 *tt*tt
   elseif (-1.0.le.tt.and.0.0.gt.tt)then
      tsq=tt*tt
      dbasis=  -2.0*tt - (3./2.)*tsq
   elseif (0.0.le.tt.and.1.0.gt.tt) then
      tsq=tt*tt
      dbasis=  -2.0*tt + (3./2.)*tsq
   elseif(1.0.le.tt.and.2.0.ge.tt)then
      tt=2.-tt
      dbasis= -0.5*tt*tt
   else
      dbasis=0.
   endif

   return
end

!--------------------------------------------------------------------

real function abasis(tl,timej,deltat)

!  find 2nd derivative of spline basis functions for knot tl, at timej, 
!  equally spaced knots deltat

!********************************************************************

      implicit real (a-h,o-z)

!********************************************************************

   tt=(timej-tl)/deltat
   if (-2.0.le.tt.and.-1.0.gt.tt)then
      tt=tt+2.
      abasis= tt
   elseif (-1.0.le.tt.and.0.0.gt.tt)then
      abasis=  - 2.0 - 3.0*tt
   elseif (0.0.le.tt.and.1.0.gt.tt) then
      abasis=  - 2.0 + 3.0*tt
   elseif(1.0.le.tt.and.2.0.ge.tt)then
      tt=tt-2.
      abasis= -tt
   else
      abasis=0.
   endif

   return

end

!--------------------------------------------------------------------

subroutine pen_matrix(nknot,h)

!
!  subroutine to construct second derivative penalty matrix.
!  This is done by analitically construct the Gram Matrix of 
!  the second derivative cubic B-splines. Note the banded 
!  structure of this matrix, with the 3rd diagonal being 0.0
!  but the fourth diagonal 1./6.
!
!  This code is based on Cathy Constable's cubic spline codes
!	C. G. Constable and R. L. Parker (1988)
!	Smoothing, Splines and Smoothing Splines; Their 
!	Applicatio in Geomagnetism.
!	Journal of Computational Physics, Vol 78, No.2   
!

!********************************************************************

      implicit none

      integer  :: nknot
      real(4), dimension(nknot,nknot) :: h

      integer :: i, j, k

!********************************************************************

!     construct the 2x2 matrices at each corner of h and h(3,2),h(2,3)
!     minimum h matrix size is 7

   if (nknot < 7) then
      write(6,*) 'Error in pen_matrix: Matrix size should be > 7'
      stop
   endif

!     intialise h

   h = 0.0


   h(1,1)=12.
   h(nknot,nknot)=12.
   h(1,2)=-18.
   h(2,1)=-18.
   h(nknot-1,nknot)=-18.
   h(nknot,nknot-1)=-18.

   h(2,2)=48.
   h(nknot-1,nknot-1)=48.
   h(3,2)=-36.
   h(2,3)=-36.

   h(3,3)=84.
   h(3,4)=-54.
   h(4,3)=-54.
   h(nknot-2,nknot-1)=-36.
   h(nknot-1,nknot-2)=-36.

   h(nknot-2,nknot-2)=84.
   h(nknot-2,nknot-3)=-54.
   h(nknot-3,nknot-2)=-54.

   do i=4,nknot-3
      do j=4,nknot-3

         k=abs(i-j)
         if (k.eq.0) then
            h(i,j)=96.
         elseif (k.eq.1) then
            h(i,j)=-54.
         endif

      enddo
   enddo
 
   do i=1,nknot
      do j=1,nknot
         
         k=abs(i-j)
         if (k.eq.3) then
            h(i,j)=6.
         endif

      enddo
   enddo

   h = h/36.		! Maintain normalization for unit splines,

   return

end subroutine pen_matrix 


!--------------------------------------------------------------------


