subroutine df_spec ( npts,dt,xi,xj,tbp,kspec,nf,                   &
                     freq,df_spectra,df_cohe,df_phase )  
            

!
!  Construct the dual-frequency spectrum from the yk's and the 
!  weights of the usual multitaper spectrum estimation. 
!  The dual-frequency spectrum is saved in a file, needed for 
!  long time series where computer memory breaks down.
!
!  Note this code uses the real(4) multitaper code. 
!
!  Modified
!	German Prieto
!	September 2005
!  

!********************************************************************

   use spectra

   implicit none

!  Inputs

   integer, intent(in) :: npts, kspec, nf

   real(4), intent(in) :: dt, tbp

   real(4), dimension(npts), intent(in) :: xi, xj

!  Outputs

   real(4), intent(out), dimension(nf) :: freq

   real(4), intent(out), dimension(nf,nf)    :: df_cohe, df_phase
 
   complex(4), intent(out), dimension(nf,nf) :: df_spectra

!  spectra and frequency

   real(4), dimension(nf) :: spec_i, spec_j

   real(4), dimension(nf,kspec) :: wt_i, wt_j 

   complex(4), dimension(npts,kspec) :: yk_i, yk_j

!  Dual freq matrices

   complex(4), dimension(nf,kspec)     :: dyk_i, dyk_j 
   real(4), dimension(nf)              :: dk_i, dk_j 
   
!  Others

   integer :: i, j

   real(4), parameter :: pi = 3.14159265358979

!********************************************************************

   write(6,'(a,2i5)') 'Data points and frequency points', npts, nf

!
!  Get the spectrum estimate
!  if xi is xj, then just do one estimate of the spectrum
!

   if (all(xi==xj)) then    
      call mtspec( npts,dt,xi,tbp,kspec,nf,freq,          &
   	            spec_i,yk=yk_i,wt=wt_i)
      spec_j = spec_i
      yk_j = yk_i
      wt_j = wt_i
   else
      call mtspec( npts,dt,xi,tbp,kspec,nf,freq,          &
                    spec_i,yk=yk_i,wt=wt_i)

      call mtspec( npts,dt,xj,tbp,kspec,nf,freq,          &
    		    spec_j,yk=yk_j,wt=wt_j)
   endif

!
!  Create the spectra (cannot use spec output, normalized different)
!

   spec_i = sum( wt_i**2 * (abs(yk_i(1:nf,:)))**2 , dim=2)/    &
                     sum((wt_i**2),dim=2)

   spec_j = sum( wt_j**2 * (abs(yk_j(1:nf,:)))**2 , dim=2)/    &
                     sum((wt_j**2),dim=2)

   dk_i = sqrt( sum (wt_i(:,:)**2, dim=2) )
   dk_j = sqrt( sum (wt_j(:,:)**2, dim=2) )

   dyk_i(:,:) = wt_i(:,:) * yk_i(1:nf,:)
   dyk_j(:,:) = wt_j(:,:) * yk_j(1:nf,:)

   do i=1,nf

      if (mod(i,1000) == 0) then
         write(6,'(a,i5,a,i5)') 'Loop ',i , ' of ', nf 
      endif

   ! df_spectrum and coherence
   
      do j = 1,nf
         
         df_spectra(i,j) = sum ( dyk_i(i,:) * conjg(dyk_j(j,:)) )   &
                               / (dk_i(i)*dk_j(j))

         df_cohe(i,j) = (abs(df_spectra(i,j)))**2 / (spec_i(i)*spec_j(j))

         df_phase(i,j) = atan2( aimag(df_spectra(i,j)), real(df_spectra(i,j)) ) 

      enddo
      
   enddo

end subroutine df_spec








