subroutine dftowv(nf,npts,df_spec,wv_spec)

!
!  Modified
!	German Prieto
!	March 2005
!
!  Rotate the dual-frequency spectrum, 45 degrees, inverse
!  FFT on one dimension (columns) and output the Wigner-Ville
!  spectrum.
!

!********************************************************************

   implicit none

   integer, intent(in) :: nf,npts

   complex(8), dimension(nf,nf), intent(in) :: df_spec

   complex(8), dimension(npts,nf), intent(out) :: wv_spec

!********************************************************************

!  Rotate the matrix

   call mat_rot(nf,npts,df_spec,wv_spec)

!  Take inverse FFT in 2D

   call ifft_2d(wv_spec,npts,nf)

   return

end subroutine dftowv

!--------------------------------------------------------------------
! Matrix Rotation
!--------------------------------------------------------------------

subroutine mat_rot(xn,tn,x,x2)

!********************************************************************

   implicit none

   integer, intent(in) :: xn, tn

   complex(8), dimension(xn,xn), intent(in)  :: x
   complex(8), dimension(tn,xn), intent(out) :: x2

   integer :: k, i
   integer :: indices, ncol, gmax

!********************************************************************

   if (xn>7000 .and. xn<=10000) then
      write(6,'(a)') 'This may take very long, up to 10 min.'
   elseif (xn>10000) then
      write(6,'(a)') '**** Too large matrix, stopped ****'
      stop
   endif

   do k = 1,xn
      gmax = min(k-1,xn-k,nint(xn/2.)-1)
    
      ncol = k
      do i = -gmax,gmax,1

         indices = mod(tn+i,tn)+1

!	 print *, 'original ', ncol+i, ncol-i
!	 print *, 'output   ', indices, ncol

!	 x2(indices,ncol) = x(ncol+i,ncol-i)
        x2(indices,ncol) = x(ncol-i,ncol+i)

      enddo

   enddo
   
   return

end subroutine mat_rot


   

