   subroutine zqrfac4(m, n,a_in, q, r)

!$$$$  calls no other routines
!  Forms a QR decomposition of a = q*r
!  Method - modified Gram-Schmidt.
!  See Golub and Van Loan: Matrix Computations (p 152)
!  m, n dimension of a 
!  a    matrix ---- overwritten with q array with n orthogonal cols
!  r    upper triangle such that a = q*r
!

!********************************************************************

   implicit none

!  Input

   integer, intent(in) :: m, n
   complex(4), dimension(m,n), intent(in) :: a_in

!  Output

   complex(4), dimension(m,n), intent(out) :: q 
   complex(4), dimension(n,n), intent(out) :: r

!  Working variables

   integer :: i, k, j
   complex(4), dimension(m,n) :: a

!********************************************************************

   a(:,:) = a_in(:,:)		! Avoid overwriting the input matrix

   do k=1,n

!  find constants for rotation and diagonal entry

      r(k,k)=0.00
      do i=1,m
         r(k,k)=conjg(a(i,k))* a(i,k) + r(k,k)
      enddo
      r(k,k)=sqrt(r(k,k))

      do i=1,m
         a(i,k) = a(i,k)/r(k,k)
      enddo
      do j=k+1,n
         r(k,j) = 0.0
         do i=1,m
            r(k,j) =conjg(a(i,k))*a(i,j) + r(k,j)
         enddo
         do i=1,m
            a(i,j) = a(i,j) - a(i,k)*r(k,j)
         enddo
      enddo
   enddo
   
   q = a

   return
      
end subroutine zqrfac4 

!--------------------------------------------------------------------

   subroutine zqrfac(m, n,a_in, q, r)

!$$$$  calls no other routines
!  Forms a QR decomposition of a = q*r
!  Method - modified Gram-Schmidt.
!  See Golub and Van Loan: Matrix Computations (p 152)
!  m, n dimension of a 
!  a    matrix ---- overwritten with q array with n orthogonal cols
!  r    upper triangle such that a = q*r
!

!********************************************************************

   implicit none

!  Input

   integer, intent(in) :: m, n
   complex(8), dimension(m,n), intent(in) :: a_in

!  Output

   complex(8), dimension(m,n), intent(out) :: q 
   complex(8), dimension(n,n), intent(out) :: r

!  Working variables

   integer :: i, k, j
   complex(8), dimension(m,n) :: a

!********************************************************************

   a(:,:) = a_in(:,:)		! Avoid overwriting the input matrix

   do k=1,n

!  find constants for rotation and diagonal entry

      r(k,k)=0.d0
      do i=1,m
         r(k,k)=conjg(a(i,k))* a(i,k) + r(k,k)
      enddo
      r(k,k)=sqrt(r(k,k))

      do i=1,m
         a(i,k) = a(i,k)/r(k,k)
      enddo
      do j=k+1,n
         r(k,j) = 0.d0
         do i=1,m
            r(k,j) =conjg(a(i,k))*a(i,j) + r(k,j)
         enddo
         do i=1,m
            a(i,j) = a(i,j) - a(i,k)*r(k,j)
         enddo
      enddo
   enddo
   
   q = a

   return
      
end subroutine zqrfac 

!--------------------------------------------------------------------



