program df_test

!
!  Testing the dual-frequency spectrum, as in 
!  	Mellors, R. Vernon, F. and Thomson, D. J. (1998) 
!  	Detection of dispersive signals using multitaper
!	dual frequency coherence
!	Geophys. J. Int. Vol 135 pp. 146-154
!  Coherence normalized to be unity on the main diagonal, if
!  auto-coherence is used.
!
!  Here matrices are saved, because too large data, crashes 
!  the code, using too much memory. Its dependent on the 
!  machine used, so, this is the save code, no matrices made,
!  just saved, ready to be loaded in Matlab, to be plotted.
!
!  Modified
! 	German Prieto
!	May 2005
!


!********************************************************************

!   use plot

   implicit none

   integer, parameter :: npts=601, nf=1+npts/2

!  time series

   real(4), dimension(npts) :: t

   real(4), dimension(npts) :: xa, xb, xc, xr

   real(4) :: fa, fb, fc

!  Spectral parameters

   integer :: kspec

   real(4) :: tbp, dt

!  dual-freq variables

   real(4), dimension(nf)       :: freq
   real(4), dimension(nf,nf)    :: dfcohe, dfphase

   complex(4), dimension(nf,nf) :: dfspec

!  Others

   integer :: i

   character (len = 100) :: output, fmt

   real(4), parameter :: pi = 3.14159265358979 


!********************************************************************

   tbp = 4.0
   kspec = 7
   dt = 1.0 

   if (npts <= 1) then
      write(6,'(a)') '>>> too few terms in the series to estimate a spectrum'
      stop
   endif

!
!  define time axis
!

   t = dble( (/(i-1, i=1,npts)/) ) * dt

   fa = 0.
   fb = 0.075
   fc = 0.050

   call random_number(xr)	
   xa = sin( 2.d0*pi*t*(fa + (fb-fa)*t/600) ) 
   xa = xa + xr

   call random_number(xr)	
   xb = sin( 2.d0*pi*t*(fa + (fc-fa)*t/600) )
   xb = xb + xr

   call random_number(xr)	
   xc = 0.5 * sin( 2.d0*pi*t*(fa + (fb-fa)*t/600) ) + 	          &
        0.5 * sin( (2.5d0)*2.d0*pi*t*(fa + (fb-fa)*t/600) )
   xc = xc + xr

   print *, 'Calling the codes'

   write(fmt,'(i12)') nf
   fmt = '(' // trim(adjustl(fmt)) // 'E16.7)'

   output = 'df_test1'
   call df_spec(npts,dt,xa,xa,tbp,kspec,nf,freq,dfspec,dfcohe,dfphase)

   open(12,file=output,form='formatted')
   do i=1,nf
      write(12,fmt) ( real(dfcohe(i,:)) )
   enddo
   close(12)

   output = 'df_test2'
   call df_spec(npts,dt,xa,xb,tbp,kspec,nf,freq,dfspec,dfcohe,dfphase)

   open(12,file=output,form='formatted')
   do i=1,nf
      write(12,fmt) ( real(dfcohe(i,:)) )
   enddo
   close(12)

   output = 'df_test3'
   call df_spec(npts,dt,xc,xc,tbp,kspec,nf,freq,dfspec,dfcohe,dfphase)

   open(12,file=output,form='formatted')
   do i=1,nf
      write(12,fmt) ( real(dfcohe(i,:)) )
   enddo
   close(12)

end program df_test








