program sine_test

! 
! The program calls the sine_psd subroutine to get the
! spectrum.  
!

!**********************************************************************

   use spectra 

   implicit none

!  time series

   integer :: npts
   real(4), dimension(:), allocatable :: x, t
   
!  spectra and frequency

   integer :: nf

   real(4), dimension(:), allocatable :: spec, freq
   integer, dimension(:), allocatable :: kopt


   real(4), dimension(:,:), allocatable :: err

!  Spectral parameters

   integer :: kspec

   real(4) :: tbp, dt
  
!  Degrees of freedom

!  Others

   integer :: ios
   
   integer :: i, k

   character (len = 100) :: filename

!********************************************************************

   kspec = 8
   tbp   = 5.
   npts  = 360

   dt = 5. 

   if (npts <= 1) then
      write(6,'(a)') '>>> Too few terms in the series to estimate a spectrum'
      stop
   endif

      
   nf = 1 + npts/2

   write(6,'(a,2i10)') 'Number of points in time series and fft ', npts, nf

!
!  Allocate memory
!

   allocate(x(npts))
   allocate(t(npts))
   allocate(freq(nf))
   allocate(spec(nf))
   allocate(err(nf,2))
   allocate(kopt(nf))


!
!  Define time axis
!

   do i = 1,npts
      t(i) = real(i)*dt
   enddo

   filename = 'th_data.dat'
   open (11, file=filename, status='old')

   do k = 1, npts
      read(11,*, iostat=ios) x(k)
      if (ios<0) then
         ! End of file encountered
         exit
      endif
   enddo

   close(11)

!
!  Get the spectrum estimate
!

   call sine_psd(npts,dt,x,0,3,0,nf,freq,spec,kopt,err)

   
   open(12,file='sine_spec.dat')
   write(6,'(4a16)') 'freq', 'spec', '-95%', '+95%'
   do i = 1,nf
      write(12,'(4E16.7)') freq(i), spec(i), err(i,1), err(i,2)
   enddo

   close(12)
   
end program sine_test

